"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadMasterKeysFromSettings = exports.generateMasterKeyAndEnableEncryption = exports.toggleAndSetupEncryption = exports.setupAndDisableEncryption = exports.setupAndEnableEncryption = void 0;
const Logger_1 = require("../../Logger");
const BaseItem_1 = require("../../models/BaseItem");
const MasterKey_1 = require("../../models/MasterKey");
const Setting_1 = require("../../models/Setting");
const EncryptionService_1 = require("../EncryptionService");
const syncInfoUtils_1 = require("../synchronizer/syncInfoUtils");
const logger = Logger_1.default.create('e2ee/utils');
function setupAndEnableEncryption(service, masterKey = null, password = null) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!masterKey) {
            // May happen for example if there are master keys in info.json but none
            // of them is set as active. But in fact, unless there is a bug in the
            // application, this shouldn't happen.
            logger.warn('Setting up E2EE without a master key - user will need to either generate one or select one of the existing ones as active');
        }
        syncInfoUtils_1.setEncryptionEnabled(true, masterKey ? masterKey.id : null);
        if (masterKey && password) {
            const passwordCache = Setting_1.default.value('encryption.passwordCache');
            passwordCache[masterKey.id] = password;
            Setting_1.default.setValue('encryption.passwordCache', passwordCache);
        }
        // Mark only the non-encrypted ones for sync since, if there are encrypted ones,
        // it means they come from the sync target and are already encrypted over there.
        yield BaseItem_1.default.markAllNonEncryptedForSync();
        yield loadMasterKeysFromSettings(service);
    });
}
exports.setupAndEnableEncryption = setupAndEnableEncryption;
function setupAndDisableEncryption(service) {
    return __awaiter(this, void 0, void 0, function* () {
        // Allow disabling encryption even if some items are still encrypted, because whether E2EE is enabled or disabled
        // should not affect whether items will enventually be decrypted or not (DecryptionWorker will still work as
        // long as there are encrypted items). Also even if decryption is disabled, it's possible that encrypted items
        // will still be received via synchronisation.
        syncInfoUtils_1.setEncryptionEnabled(false);
        // The only way to make sure everything gets decrypted on the sync target is
        // to re-sync everything.
        yield BaseItem_1.default.forceSyncAll();
        yield loadMasterKeysFromSettings(service);
    });
}
exports.setupAndDisableEncryption = setupAndDisableEncryption;
function toggleAndSetupEncryption(service, enabled, masterKey, password) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!enabled) {
            yield setupAndDisableEncryption(service);
        }
        else {
            if (masterKey) {
                yield setupAndEnableEncryption(service, masterKey, password);
            }
            else {
                yield generateMasterKeyAndEnableEncryption(EncryptionService_1.default.instance(), password);
            }
        }
        yield loadMasterKeysFromSettings(service);
    });
}
exports.toggleAndSetupEncryption = toggleAndSetupEncryption;
function generateMasterKeyAndEnableEncryption(service, password) {
    return __awaiter(this, void 0, void 0, function* () {
        let masterKey = yield service.generateMasterKey(password);
        masterKey = yield MasterKey_1.default.save(masterKey);
        yield setupAndEnableEncryption(service, masterKey, password);
        yield loadMasterKeysFromSettings(service);
        return masterKey;
    });
}
exports.generateMasterKeyAndEnableEncryption = generateMasterKeyAndEnableEncryption;
function loadMasterKeysFromSettings(service) {
    return __awaiter(this, void 0, void 0, function* () {
        const masterKeys = yield MasterKey_1.default.all();
        const passwords = Setting_1.default.value('encryption.passwordCache');
        const activeMasterKeyId = syncInfoUtils_1.getActiveMasterKeyId();
        logger.info(`Trying to load ${masterKeys.length} master keys...`);
        for (let i = 0; i < masterKeys.length; i++) {
            const mk = masterKeys[i];
            const password = passwords[mk.id];
            if (service.isMasterKeyLoaded(mk))
                continue;
            if (!password)
                continue;
            try {
                yield service.loadMasterKey(mk, password, activeMasterKeyId === mk.id);
            }
            catch (error) {
                logger.warn(`Cannot load master key ${mk.id}. Invalid password?`, error);
            }
        }
        logger.info(`Loaded master keys: ${service.loadedMasterKeysCount()}`);
    });
}
exports.loadMasterKeysFromSettings = loadMasterKeysFromSettings;
//# sourceMappingURL=utils.js.map